#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "buf.h"

void lnbuf_init(lnbuf_t *l, lnproc_t callback, void *cbdata)
{
	l->len = 0;
	l->alloc = 0;
	l->data = NULL;
	l->callback = callback;
	l->cbdata = cbdata;
}

void lnbuf_write(lnbuf_t *l, char *d, int n)
{
	if(l->len + n >= l->alloc) {
		l->alloc += n + 1024;
		l->data = realloc(l->data, l->alloc);
		if(!l->data) abort();
	}

	memcpy(l->data + l->len, d, n);
	l->len += n;

	for(;;) {
		char *ln = memchr(l->data, '\n', l->len);
		if(!ln) return;
		if(l->callback) l->callback(l->data, ln, l->cbdata);
		int lno = (ln - l->data);
		memmove(l->data, ln+1, l->len - (lno+1));
		l->len -= (lno+1);
	}
}

void growbuf_init(growbuf_t *b)
{
	b->alloc = 0;
	b->buf = NULL;
	b->data = NULL;
	b->cur = NULL;
}

void growbuf_alloc(growbuf_t *b, int amt)
{
	if(!b->buf) {
		b->buf = malloc(amt);
		if(!b->buf) abort();
		b->alloc = amt;
		b->data = b->buf;
		b->cur = b->buf;
	} else if(b->data + amt > b->buf + b->alloc) {
		int at = b->data - b->buf;
		int cur = b->cur - b->buf;
		b->alloc = at + amt;
		b->buf = realloc(b->buf, b->alloc);
		if(!b->buf) abort();
		b->cur = b->buf + cur;
		b->data = b->buf + at;
	}
}

void growbuf_consume(growbuf_t *b, int amt)
{
	if(!amt) return;

	assert(b->cur);
	assert(b->cur + amt <= b->data);

	b->cur += amt;
	if(b->cur == b->data) {
		b->cur = b->buf;
		b->data = b->buf;
	}
}

void growbuf_write(growbuf_t *b, void *data, int len)
{
	growbuf_alloc(b, len);
	memcpy(b->data, data, len);
	b->data += len;
}

int growbuf_hasdata(growbuf_t *b)
{
	if(b->cur) {
		return b->data - b->cur;
	} else {
		return 0;
	}
}
