#!/bin/sh
KEEPALIVE=0
DEBUGMODE=0
if [ "$1" = "--debug" ]; then
	shift
	DEBUGMODE=1
fi
if [ "$1" = "-a" ]; then
	shift
	KEEPALIVE=1
fi
if [ $# -lt 1 ]; then
	echo "Need an argument: name of plumbwrap script"
	exit 1
fi
SCRIPT="$1"
shift

CMD=`gawk -v keepalive=$KEEPALIVE -v debugmode=$DEBUGMODE -F '[[:space:]|=]+' '
function parseoptions(opt,                        words, nwords, i, cut, brk)
{
	sub(/^[[:space:]]*/, "", opt)
	sub(/[[:space:]]*$/, "", opt)
	nwords=split(opt, words)

	if(nwords == 0) return "";
	if(nwords == 1) return "-p " words[1];

	cut=0
	brk=0

	for(i=2; i<=nwords; i++) {
		if(words[i] == "cut") {
			if(cut) {
				print "Syntax error: cut specified twice" > "/dev/stderr"
				endnormal=0
				exit 1
			}
			cut = 1
		} else if(words[i] == "break") {
			if(brk) {
				print "Syntax error: break specified twice" > "/dev/stderr"
				endnormal=0
				exit 1
			}
			brk = 1
		} else {
			print "Syntax error: invalid option word", words[i] > "/dev/stderr"
			endnormal=0
			exit 1
		}
	}

	if(brk) {
		if(cut) return "-K " words[1];
		return "-k " words[1];
	} else {
		if(cut) return "-P " words[1];
		return "-p " words[1];
	}
}

function shellescape(str)
{
	gsub(/'"'"'/, "'"'"'\"'"'"'\"'"'"'", str)
	gsub(/\\\\/, "\\\\\\\\\\\\\\\\", str)
	return str
}

function getproc(name,                     i)
{
	for(i=0; i<nprocs; i++) {
		if(procs[i] == name) return i;
	}

	print "Process by name", name, "not found" > "/dev/stderr"
	endnormal=0
	exit 1
}

function parseinput(ins,                               parts, nparts, fd) {
	nparts=split(ins, parts, /\//)
	if(nparts == 1) {
		return getproc(parts[1])+1
	} else {
		fd = int(parts[2])
		if(parts[1] == "") {
			if(fd == 0) return "0";
			print "Pipe: with input /N, N can only be zero (plumb stdin)" > "/dev/stderr"
			endnormal=0
			exit 1
		}
		if(fd == 1) {
			return getproc(parts[1])+1
		} else if(fd == 2) {
			return "e" (getproc(parts[1])+1)
		} else {
			print "Pipe: Can only use fd 1 or 2 from a process as input" > "/dev/stderr"
			endnormal=0
			exit 1
		}
	}
}

function parseoutput(outs,                          parts, nparts, fd) {
	nparts=split(outs, parts, /\//)
	if(nparts == 1) {
		return getproc(parts[1])+1
	} else {
		fd = int(parts[2])
		if(parts[1] == "") {
			if(fd == 1) return "0";
			if(fd == 2) return "e";
			print "Pipe: with output /N, N can be only 1 or 2 (plumb stdout or stderr)" > "/dev/stderr"
			endnormal=0
			exit 1
		}
		if(fd == 0) {
			return getproc(parts[1])+1
		} else {
			print "Pipe: can only use fd 0 of a process as output" > "/dev/stderr"
			endnormal=0
			exit 1
		}
	}
}


BEGIN {
	append=-1
	nprocs=0
	endnormal=1
}

append == -1 && /^[[:space:]]*exec[[:space:]]+\w+[[:space:]]*=.*[^\\\\]$/ {
	sub(/^[[:space:]]*/, "")
	proc=$2
	sub(/[^=]*=[[:space:]]*/, "")
	sub(/[[:space:]]*$/, "")
	procs[nprocs]=proc
	command="+"
	for(i=1; i<=NF; i++) {
		command=command " " $i
	}
	command=command " \\\\;"
	exec[nprocs]=command
	nprocs+=1
	next
}

append == -1 && /^[[:space:]]*\w+[[:space:]]*=[[:space:]]*.*[^\\\\]$/ {
	if($1 == "") proc=$2; else proc = $1;
	sub(/[^=]*=[[:space:]]*/, "")
	command=$0
	procs[nprocs]=proc
	commands[nprocs]=command
	nprocs+=1
	next
}

append == -1 && /^[[:space:]]*\w+[[:space:]]*=[[:space:]]*.*\\\\$/ {
	if($1 == "") proc=$2; else proc = $1;
	sub(/[^=]*=[[:space:]]*/, "")
	sub(/\\\\$/, "")
	command=$0
	procs[nprocs]=proc
	commands[nprocs]=command
	append=nprocs
	nprocs+=1
	next
}

append == -1 && /^[[:space:]]*$/ {
	next
}

append == -1 && /^#/ {
	next
}

append == -1 && /^[[:space:]]*(\w+|\w*\/[0-9]+)([[:space:]]+(\w+|\w*\/[0-9]+))*[[:space:]]*\|[[:space:]]*(\w+|\w*\/[0-9]+)([[:space:]]+(\w+|\w*\/[0-9]+))*[[:space:]]*(:[[:space:]]*\w+([[:space:]]+(cut|break))*[[:space:]]*|)(\+[[:space:]]*\w+[[:space:]]*|)$/ {
	options=$0
	sub(/[^:]*(:|$)/, "", options)
	sub(/\+.*/, "", options)

	prepend=$0
	sub(/[^+]*(\+|$)/, "", prepend)
	sub(/^[[:space:]]*/, "", prepend)
	sub(/[[:space:]]*$/, "", prepend)

	sub(/:.*/, "")
	sub(/\+.*/, "")
	orig=$0

	inputs=0
	outputs=0

	sub(/\|.*/, "")
	for(i=1; i<=NF; i++) {
		if($i != "") {
			input[inputs]=parseinput($i);
			inputs+=1
		}
	}
	$0=orig
	sub(/.*\|/, "")
	for(i=1; i<=NF; i++) {
		if($i != "") {
			output[outputs]=parseoutput($i);
			outputs+=1
		}
	}

	options = parseoptions(options)

	for(i=0; i<inputs; i++) {
		for(j=0; j<outputs; j++) {
			pipes = pipes " " input[i] " " options " " output[j]
			if(prepend != "") {
				pipes = pipes " +p " prepend
			}
		}
	}

	next
}

append == -1 {
	print "Syntax unrecognized: " $0 > "/dev/stderr"
	endnormal=0
	exit 1
}

append >= 0 && /\\\\$/ {
	sub(/\\\\$/, "")
	commands[append] = commands[append] $0
	next
}

append >= 0 {
	commands[append] = commands[append] $0
	append = -1
	next
}

END {
	if(!endnormal) exit 1;

	if(append >= 0) {
		print "Last line cannot end with \\\\" > "/dev/stderr"
		exit 1
	}

	printf "plumb"
	if(debugmode) printf " --debug";
	if(keepalive) printf " -a";

	for(i=0; i<nprocs; i++) {
		if(i in commands) {
			printf " : '"'"'%s'"'"'", shellescape(commands[i])
		} else if(i in exec) {
			printf " %s", exec[i]
		} else {
			print "whoops, internal error" > "/dev/stderr"
			exit 1
		}
	}

	print pipes
}
' < "$SCRIPT"` && eval "$CMD"
